interface Response<T = Record<string, unknown>> {
  status: number,
  data?: T,
}

interface ErrorResponse extends Response {
  message: string,
}

type ResponseParams<T extends Record<string, unknown> = Record<string, unknown>> = {
  message?: string,
  data?: T,
}

export class NotFound implements ErrorResponse {
  public readonly status = 404
  public readonly message: string
  constructor(params: ResponseParams) {
    this.message = params.message
  }
}

export class Created<T extends Record<string, unknown>> implements Response {
  public readonly status = 201
  public readonly data: T
  constructor(params: ResponseParams<T>) {
    this.data = params.data
  }
}

export class Saved<T extends Record<string, unknown>> implements Response {
  public readonly status = 200
  public readonly data: T
  constructor(params: ResponseParams<T>) {
    this.data = params.data
  }
}

export class Retrieved<T extends Record<string, unknown>> implements Response {
  public readonly status = 200
  public readonly data: T
  constructor(params: ResponseParams<T>) {
    this.data = params.data
  }
}
